import ankidmpy.util as util
import csv
import os.path


def indexIt(full, base):
    filenm = os.path.join(base, 'data.csv')
    try:
        with open(filenm, newline='') as csvfile:
            reader = csv.reader(csvfile)
            header = next(reader)
            guid_column = None
            try:
                guid_column = header.index('guid')
            except ValueError:
                util.err("Missing 'guid' column")
            data = list(reader)
    except PermissionError:
        util.err("Cannot read file: %s" % (filenm,))

    guids = []
    try:
        with open(filenm, 'w', newline='') as csvfile:
            writer = csv.writer(csvfile)
            writer.writerow(header)
            for row in data:
                guid = row[guid_column]
                if not guid or (guid in guids) or full:
                    guid = util.createGuid()
                guids.append(guid)
                # FIXME: I'm assuming we're supposed to use the guid
                # row[guid_column] = guid
                writer.writerow(row)

    except PermissionError:
        util.err("Cannot write to file: %s" % (filenm,))

    util.msg("Successfully reindexed 'data.csv'")



import ankidmpy.util as util
from collections import defaultdict
import shutil
import os.path


def build(decks, src_dir, build_dir, lang):
    glbals = dict(deck=util.getJson(os.path.join(src_dir, 'deck.json')),
                  config=util.getJson(os.path.join(src_dir, 'config.json')),
                  model=util.getJson(os.path.join(src_dir, 'model.json')),
                  media=util.getFilesList(os.path.join(src_dir, 'media')),
                  templates=util.getTemplates(os.path.join(
                      src_dir, 'templates')),
                  desc=util.getRaw(os.path.join(src_dir, 'desc.html')),
                  css=util.getRaw(os.path.join(src_dir, 'style.css')),
                  data=util.getCsv(os.path.join(src_dir, 'data.csv')))

    languages = list(glbals['data'].keys())
    if lang:
        if lang not in languages:
            util.err("Language '%s' is not available." % (lang,))
        languages = [lang]

    for lang in languages:
        decks_build = _readDecks(decks, os.path.join(src_dir, 'decks'))
        for deck, deck_build in decks_build.items():
            util.msg("Building deck: %s (Language: %s)" % (deck, lang))

            deck_build['deck']['uuid'] = util.uuidEncode(
                deck_build['deck']['uuid'], lang)
            deck_build['config']['uuid'] = util.uuidEncode(
                deck_build['config']['uuid'], lang)
            deck_build['model']['uuid'] = util.uuidEncode(
                deck_build['model']['uuid'], lang)

            deck_data = {
                '__type__':
                    'Deck',
                'crowdanki_uuid':
                    deck_build['deck']['uuid'],
                'name':
                    util.filenameToDeck(deck if lang ==
                                        'default' else "%s[%s]" % (deck, lang)),
                'desc':
                    deck_build.get('@desc') or glbals['desc']
            }
            deck_data.update(glbals['deck'])
            deck_data.update(deck_build['@deck'])

            deck_data['deck_configurations'] = [{
                '__type__': 'DeckConfig',
                'crowdanki_uuid': deck_build['config']['uuid'],
                'name': deck_build['config']['name']
            }]
            deck_data['deck_configurations'][-1].update(glbals['config'])
            deck_data['deck_configurations'][-1].update(deck_build['@config'])

            deck_data['deck_config_uuid'] = deck_build['config']['uuid']

            deck_templates_info = []
            k = 0
            for template in deck_build['templates']:
                template_filename = util.ensureFilename(template)
                if not template_filename in glbals['templates']:
                    util.err("Field template '%s not found." % (template,))
                deck_templates_info.append(dict(name=template, ord=k))
                deck_templates_info[-1].update(
                    glbals['templates'][template_filename])
                k += 1

            deck_fields_info = []
            i = 0
            for field in deck_build['fields']:
                if field not in glbals['data'][lang]:
                    util.err("Field '%s' not found" % (field,))
                deck_fields_info.append(dict(name=field, ord=i))
                deck_fields_info[-1].update(util.getFieldDefaults())
                i += 1
            deck_data['note_models'] = [{
                '__type__': 'NoteModel',
                'crowdanki_uuid': deck_build['model']['uuid'],
                'name': deck_build['model']['name'],
                'flds': deck_fields_info,
                'tmpls': deck_templates_info,
                'css': deck_build.get('@css') or glbals['css']
            }]
            deck_data['note_models'][-1].update(glbals['model'])
            deck_data['note_models'][-1].update(deck_build['@model'])

            if 'guid' not in glbals['data'][lang]:
                util.err("Missed required 'guid' column in 'data.csv'")

            if len(glbals['data'][lang]['guid']) != len(
                    set(glbals['data'][lang]['guid'])):
                util.err(
                    "Found duplicate values in 'guid' column.  Run 'index' command."
                )

            deck_fields_data = defaultdict(lambda: defaultdict(list))
            deck_media = []
            for field in deck_build['fields']:
                if field not in glbals['data'][lang]:
                    util.err("Column '%s' is missing in 'data.csv'." % (field,))
                for i, cell in enumerate(glbals['data'][lang][field]):
                    deck_fields_data[i]['fields'].append(cell)
                    # FIXME: This seems like it would lead to duplicates
                    # Should it be seeing if the media_file is referenced??
                    for media_file in glbals['media']:
                        if cell in media_file:
                            deck_media.append(media_file)

            for i, cell in enumerate(glbals['data'][lang]['guid']):
                if not cell:
                    util.err("""Missing value in the 'guid' field in the row:

'index' command to fix the problem.""" %
                             (util.toJson(deck_fields_data[i]['fields']),))
                deck_fields_data[i]['guid'] = util.guidDecode(
                    cell, deck_build['model']['uuid'])

            if 'tags' in glbals['data'][lang]:
                for i, cell in enumerate(glbals['data'][lang]['tags']):
                    deck_fields_data[i]['tags'] = cell.split(' ')

            deck_data['media_files'] = deck_media
            deck_data['notes'] = []
            for i, row in sorted(deck_fields_data.items(), key=lambda x: x[0]):
                deck_data['notes'].append({
                    '__type__': 'Note',
                    'data': '',
                    'fields': row['fields'],
                    'flags': 0,
                    'guid': row['guid'],
                    'note_model_uuid': deck_build['model']['uuid'],
                    'tags': row.get('tags') or []
                })

            localized_deck = deck if lang == 'default' else '_'.join(
                (deck, lang))
            build_dir = build_dir or 'build'
            deck_dir = os.path.join(build_dir, localized_deck)
            util.prepareDir(deck_dir)
            with open(os.path.join(deck_dir, localized_deck + '.json'),
                      'w') as f:
                f.write(util.toJson(deck_data))

            util.prepareDir(os.path.join(deck_dir, 'media'))
            for media_file in deck_media:
                shutil.copy(os.path.join(src_dir, 'media', media_file),
                            os.path.join(deck_dir, 'media', media_file))


def _readDecks(decks, directory):
    decks_data = dict()

    if not decks:
        decks = util.getFilesList(directory)

    for deck in decks:
        deck_filename = util.deckToFilename(deck)
    dirnm = os.path.join(directory, deck_filename)
    if os.path.exists(dirnm) and os.path.isdir(dirnm):
        decks_data[deck_filename] = _readDeck(dirnm)
    else:
        util.err("Deck not found: %s" % (dirnm,))

    return decks_data


def _readDeck(directory):
    inDir = lambda fn: os.path.join(directory, fn)
    deck_data = util.getJson(inDir('build.json'), required=False)
    deck_data['@deck'] = util.getJson(inDir('deck.json'), required=False)
    deck_data['@config'] = util.getJson(inDir('config.json'), required=False)
    deck_data['@desc'] = util.getRaw(inDir('info.html'), required=False)
    deck_data['@model'] = util.getJson(inDir('model.json'), required=False)
    deck_data['@css'] = util.getRaw(inDir('style.css'), required=False)
    return deck_data



import ankidmpy.builder as builder
import ankidmpy.copier as copier
import ankidmpy.indexer as indexer
import ankidmpy.util as util
import ankidmpy.importer as importer
import os.path
import argparse
import sys

DIRNAME, _ = os.path.split(__file__)
TEMPLATES_DIR = os.path.abspath(os.path.join(DIRNAME, 'templates'))
templates = util.getFilesList(TEMPLATES_DIR, 'dir')


def initDeck(args):
    template = args.template
    if not template in templates:
        util.err('Cannot find template: %s' % (template,))

    util.prepareDir(args.base)

    if not util.isDirEmpty(args.base):
        util.err("Directory '%s' is not empty." % (args.base,))

    importer.importIt(os.path.join(TEMPLATES_DIR, template), args.base,
                      args.deck)


def importDeck(args):
    util.prepareDir(args.base)

    if not util.isDirEmpty(args.base):
        util.err("Directory '%s' is not empty." % (args.base,))

    importer.importIt(args.path, args.base, args.deck)


def buildDeck(args):
    builder.build(args.deck, args.base, args.build, args.lang)


def indexDeck(args):
    indexer.indexIt(args.full, args.base)


def copyDeck(args):
    copier.copy(args.deck1, args.deck2, args.base)


def parse_arguments():
    DESCRIPTION = """
    This tool disassembles CrowdAnki decks into collections of files
    and directories which are easy to maintain. It then allows you to can
    create variants of your deck via combining fields, templates and data 
    that you really need. You can also use this tool to create translations
    of your deck by creating localized columns in data files.
    """
    parser = argparse.ArgumentParser(prog="anki-dm", description=DESCRIPTION)
    parser.set_defaults(command=None)

    subparsers = parser.add_subparsers()

    parser_init = subparsers.add_parser(
        'init', help="Create a new deck from a template.")
    parser_init.add_argument('template',
                             help='Template to use when creating the deck set.')
    parser_init.add_argument(
        '--deck',
        dest='deck',
        help='''Name of the default deck of the deck set being created.
                          If not provided, then the original deck/template name will be used.'''
    )
    parser_init.set_defaults(command=initDeck)

    parser_import = subparsers.add_parser(
        'import', help="Import a CrowdAnki deck to Anki-dm format")
    parser_import.add_argument(
        'path', help='Path to a CrowdAnki deck directory to import.')
    parser_import.add_argument(
        '--deck',
        dest='deck',
        help='''Name of the default deck of the deck set being created.
                          If not provided, then the original deck/template name will be used.'''
    )
    parser_import.set_defaults(command=importDeck)

    parser_build = subparsers.add_parser(
        'build', help="Build Anki-dm deck into CrowdAnki format")
    parser_build.add_argument(
        'deck',
        nargs='*',
        help=
        'Decks to build. If not specified then all decks of the deck set will be built.'
    )
    parser_build.add_argument(
        '--lang',
        dest='lang',
        help='''Build decks for the specific language code.  
                          If omitted then decks for all languages will be built.'''
    )
    parser_build.add_argument('--build',
                              dest='build',
                              help='''Path to the build directory.
                          [Default: build]''')
    parser_build.set_defaults(command=buildDeck)

    parser_copy = subparsers.add_parser(
        'copy', help='Make reindexed copy of Anki-dm deck.')
    parser_copy.add_argument('deck1', help="Source deck")
    parser_copy.add_argument(
        'deck2',
        nargs='?',
        help=
        '''Destination deck. If not provided, this is calculated automatically by suffixing'''
    )
    parser_copy.set_defaults(command=copyDeck)

    parser_index = subparsers.add_parser(
        'index', help="Set guids for rows missing them.")
    parser_index.add_argument('--full',
                              dest='full',
                              action='store_true',
                              help='Reindex all data rows.')
    parser_index.set_defaults(command=indexDeck)

    parser.add_argument('--base',
                        dest='base',
                        default=".",
                        help='''Path to the deck set directory.
                          [Default: src]''')
    parser.add_argument('--templates',
                        dest='templates',
                        action='store_true',
                        help='List all available templates.')

    return parser.parse_args()


def main():

    args = parse_arguments()

    if args.templates:
        if len(templates):
            util.msg('\n'.join(templates))
            sys.exit(0)
        else:
            util.err("No templates found")

    if args.command:
        args.command(args)



import csv
import json
import re
from collections import defaultdict
import uuid
import random
import os.path
import sys

GUID_CHARS = 'abcdefghijklmnopqrstuvwxyz' + 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' + '0123456789' + "!#$%&()*+,-./:;<=>?@[]^_`{|}~"


def prepareDir(directory):
    if not os.path.exists(directory):
        os.makedirs(directory, 0o755)
    if not os.path.isdir(directory):
        raise RuntimeError("Cannot create directory: %s" % (directory,))


def msg(msg):
    print(msg)


def err(msg):
    raise RuntimeError(msg)


def warn(msg):
    print(msg, file=sys.stderr)


def toJson(data):
    res = json.dumps(data, indent=2, ensure_ascii=False)
    return re.sub(r'/^(  +?)\\1(?=[^ ])/m', '\1', res)


def getFilesList(directory, typ='file'):
    data = []
    try:
        for fn in os.listdir(directory):
            if fn in ('.', '..'):
                continue
            if typ == 'file' and os.path.isfile(os.path.join(directory, fn)):
                data.append(fn)
            elif typ == 'dir' and os.path.isdir(os.path.join(directory, fn)):
                data.append(fn)
            else:
                data.append(fn)
    except:
        pass

    return data


def getJsons(directory):
    return [
        getJson(os.path.join(directory, fn)) for fn in getFilesList(directory)
    ]


def getFields(directory):
    return dict((base, getJson(os.path.join(directory, base + ext)))
                for base, ext in map(os.path.splitext, getFilesList(directory))
                if ext == '.json')


def getTemplates(directory):
    data = dict()
    for fn in getFilesList(directory):
        base, ext = os.path.splitext(fn)
        if ext != '.html':
            continue
        html = getRaw(os.path.join(directory, fn))
        lines = html.splitlines()
        try:
            idx = lines.index('--')
        except ValueError:
            raise RuntimeError(
                "Incorrect template: %s.  It must consist of two parts divided by '--' on a separate line."
            )
        data[base] = dict(qfmt='\n'.join(lines[:idx]),
                          afmt='\n'.join(lines[idx + 1:]),
                          name=base,
                          bafmt='',
                          bqfmt='',
                          did=None)
    return data


def getRaw(fn, required=True):
    if not required and not os.path.exists(fn):
        return None
    with open(fn) as f:
        contents = f.read()
    return contents


def getFieldDefaults():
    return dict(font='Arial', media=[], rtl=False, size=20, sticky=False)


def getJson(fn, required=True):
    data = getRaw(fn, required)
    if not data:
        return []
    return json.loads(data)


def getCsv(fn, required=True):
    if not required and not os.path.exists(fn):
        return None

    langs = defaultdict(dict)
    result = defaultdict(lambda: defaultdict(list))
    with open(fn, newline='') as csvfile:
        reader = csv.reader(csvfile)
        header = next(reader)
        for i, col in enumerate(header):
            if ':' in col:
                field, lang = col.rsplit(':', 1)
                if field == 'guid':
                    warn('Translating "guid" field doesn\'t make any sense.')
                langs[lang][field] = i
            else:
                langs['default'][col] = i

        for lang, cols in langs.items():
            if lang != 'default':
                langs[lang].update(langs['default'])

            langs[lang] = cols = dict(
                reversed(pr) for pr in langs[lang].items())

        for row in reader:
            for lang, cols in langs.items():
                for i, cell in enumerate(row):
                    if i in cols:
                        result[lang][cols[i]].append(cell)

    if not result:
        # Create one row anyway
        for lang, cols in langs.items():
            for i, col in enumerate(cols):
                result[lang][col].append('')

    return result


def createGuid():
    table = GUID_CHARS
    num = random.randint(0, 2**63)
    buf = ""
    while num:
        mod = num % len(table)
        num = int((num - mod) / len(table))
        buf = table[mod] + buf
    return buf


def createUuid():
    return str(uuid.uuid1())


def uuidEncode(uuid, lang):
    if lang == 'default':
        return uuid
    table = '0123456789abcdef'
    lang_code = sum(ord[c] for c in lang)

    result = ''
    for a in uuid:
        if a == '-':
            result += '-'
            continue
        try:
            an = table.index(a)
        except:
            err("Cannot encode 'uuid': uuid char not found: %s. 'lang' = %s, 'uuid' = %s"
                % (a, lang, uuid))
        cn = an + lang_code % len(table)
        result += table[cn]

    return result


def guidEncode(guid, uuid):
    return _guidTransform(guid, uuid, 'encode')


def guidDecode(guid, uuid):
    return _guidTransform(guid, uuid, 'decode')


def _guidTransform(guid, uuid, direction='encode'):
    table = GUID_CHARS
    i = 0
    result = ''
    for b in uuid:
        cn = 0
        a = guid[i]
        try:
            an = table.index(a)
        except:
            err("Cannot encode 'guid': guid char not found: %s.  'guid' = %s, 'uuid' = %s"
                % (a, guid, uuid))
        try:
            bn = table.index(b)
        except:
            err("Cannot encode 'guid': guid char not found: %s.  'guid' = %s, 'uuid' = %s"
                % (b, guid, uuid))

        if dir == 'encode':
            cn = an - bn
        else:
            cn = an + bn

        if cn >= len(table):
            cn = cn % len(table)
        elif cn < 0:
            cn = len(table) + cn

        result += table[cn]
        i = (i + 1) % len(guid)

    # FIXME: Original code wrapped around and overwrote beginning if len(guid) < len(uuid)
    # Is this just a makeshift hash?  I guess we check that the generated hashes are unique
    # afterward.  I guess you roll the dice when you reindex.  GUID seems to be a misnomer.

    rln = len(result)
    gln = len(guid)
    if rln < gln:
        return result
    if rln % gln == 0:
        return result[-gln:]

    split = (int(rln / gln)) * gln
    return result[split:] + result[-gln:split]


def isDirEmpty(directory):
    try:
        return len(os.listdir(directory)) - 2
    except:
        return None


def deckToFilename(deck):
    filename = deck.replace('_', '___')
    filename = filename.replace('::', '__')
    return ensureFilename(filename)


def filenameToDeck(filename):
    deck = re.sub(r'(_+)', lambda m: '::' if len(m[1]) == 2 else m[1], filename)
    deck = deck.replace('___', '_')
    return deck


def ensureFilename(filename):
    allowed_chars = 'abcdefghijklmnopqrstuvwxyz' + 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' + '0123456789' + "$-_ "
    result = ''
    for char in filename:
        result += char if char in allowed_chars else '-'

    return result


def checkFieldName(name):
    if name in ('guid', 'tags'):
        err("Fields with names 'guid' and 'tags' are reserved.  Please rename them first."
           )
    return name



import ankidmpy.util as util
import shutil
import os.path


def copy(deck1, deck2, base):
    deck1_path = os.path.join(base, 'decks', util.deckToFilename(deck1))

    if not os.path.isdir(deck1_path):
        util.err("Source deck not found: %s" % (deck1,))

    deck2_suffix = ""
    if deck2:
        deck2_path = os.path.join(base, 'decks', util.deckToFilename(deck2))
    else:
        i = 1
        while True:
            deck2_suffix = " (%d)" % (i,)
            deck2_path = deck1_path + deck2_suffix
            if not os.path.exists(deck2_path):
                break
            i += 1

    try:
        shutil.copytree(deck1_path, deck2_path)
    except PermissionError:
        util.err("Cannot copy files")

    deck_build = util.getJson(os.path.join(deck2_path, 'build.json'))

    deck_build['deck']['uuid'] = util.createUuid()
    deck_build['config']['uuid'] = util.createUuid()
    deck_build['model']['uuid'] = util.createUuid()

    with open(os.path.join(deck2_path, 'build.json'), 'w') as f:
        f.write(util.toJson(deck_build))

    util.msg("Created deck: %s" % deck2 if deck2 else deck1 + deck2_suffix)



import ankidmpy.util as util
from collections import defaultdict
import shutil
import csv
import os.path


def importIt(path, directory, deck=None):
    path = path.rstrip('/')
    _, basename = os.path.split(path)
    filenm = os.path.join(path, basename + '.json')
    if not os.path.exists(filenm):
        filenm = os.path.join(path, 'deck.json')

    directory = directory.rstrip('/')
    if not directory:
        directory = 'src'

    build_info = defaultdict(dict)

    deck_data = util.getJson(filenm)

    if len(deck_data['deck_configurations']) > 1 or len(
            deck_data['note_models']) > 1:
        util.err("Multiple models or configurations per deck is not supported")

    if len(deck_data['deck_configurations']) == 0 or len(
            deck_data['note_models']) == 0:
        util.err(
            "Decks with empty models or configurations are note supported.  Try adding one card in your deck."
        )

    build_info['deck']['uuid'] = util.createUuid()

    dictSlice = lambda d, kys: {key: d[key] for key in d.keys() & kys}

    deck_info = dictSlice(deck_data, {'dyn', 'extendNew', 'extendRev'})
    deck_info['children'] = []
    with open(os.path.join(directory, 'deck.json'), 'w') as f:
        f.write(util.toJson(deck_info))

    configuration = deck_data['deck_configurations'][0]
    build_info['config']['uuid'] = util.createUuid()
    build_info['config']['name'] = configuration['name']
    configuration_info = dictSlice(configuration, {
        'autoplay', 'dyn', 'lapse', 'maxTaken', 'new', 'replayq', 'rev', 'timer'
    })
    with open(os.path.join(directory, 'config.json'), 'w') as f:
        f.write(util.toJson(configuration_info))

    desc = deck_data['desc']
    with open(os.path.join(directory, 'desc.html'), 'w') as f:
        f.write(desc)

    # FIXME: This is the requirement that there be only one model
    model = deck_data['note_models'][0]
    build_info['model']['uuid'] = util.createUuid()
    build_info['model']['name'] = model['name']
    model_info = dictSlice(model, {'latexPost', 'latexPre', 'type'})
    model_info['vers'] = []

    with open(os.path.join(directory, 'model.json'), 'w') as f:
        f.write(util.toJson(model_info))

    field_list = [v['name'] for v in model['flds']]

    notes = deck_data['notes']
    header = ['guid'] + field_list + ['tags']
    csvfn = os.path.join(directory, 'data.csv')
    try:
        with open(csvfn, 'w', newline='') as csvfile:
            writer = csv.writer(csvfile)
            writer.writerow(header)
            for note in notes:
                row = [
                    util.guidEncode(note['guid'], build_info['model']['uuid'])
                ]

                for i, field in enumerate(field_list):
                    row.append(note['fields'][i])
                row.append(' '.join(note['tags']))

                writer.writerow(row)

    except PermissionError:
        util.err("Cannot write to file: %s" % (csvfn,))

    media_files = deck_data['media_files']
    util.prepareDir(os.path.join(directory, 'media'))
    for media_file in media_files:
        shutil.copy(os.path.join(path, 'media', media_file),
                    os.path.join(directory, 'media', media_file))

    templates = model['tmpls']
    fulltemplateDirname = os.path.join(directory, 'templates')
    util.prepareDir(fulltemplateDirname)
    for template in templates:
        template_filename = util.ensureFilename(template['name'])
        with open(
                os.path.join(fulltemplateDirname, template_filename + '.html'),
                'w') as f:
            f.write(template['qfmt'])
            f.write("\n\n--\n\n")
            f.write(template['afmt'])

    template_list = [v['name'] for v in templates]

    css = model['css']
    with open(os.path.join(directory, 'style.css'), 'w') as f:
        f.write(css + '\n')

    if deck:
        deck_name = deck
    else:
        deck_name = deck_data['name']
        deck = deck_name

    deck_dir_name = util.deckToFilename(deck_name)
    fulldirname = os.path.join(directory, 'decks', deck_dir_name)
    util.prepareDir(fulldirname)

    build_info.update({'fields': field_list, 'templates': template_list})
    with open(os.path.join(fulldirname, 'build.json'), 'w') as f:
        f.write(util.toJson(build_info))

    util.msg("Created deck: %s" % (deck,))



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['ankidmpy']

package_data = \
{'': ['*'], 'ankidmpy': ['templates/Default/*']}

entry_points = \
{'console_scripts': ['anki-dm = ankidmpy:main']}

setup_kwargs = {
    'name': 'ankidmpy',
    'version': '0.1.1',
    'description': 'Python port of github.com/OnkelTem/anki-dm',
    'long_description': '# **ankidmpy**\n\n**ankidmpy** ( pronounced "anki-dumpy" ) is a straightforward port of [anki-dm](https://github.com/OnkelTem/anki-dm)    to `python`.   The original **anki-dm** is written in `PHP` and is a tool to work with the [CrowdAnki plugin](https://github.com/Stvad/CrowdAnki) for the [Anki](https://apps.ankiweb.net/) spaced repetition memory app to facilitate collaborative building of flash card decks. \n\n## Overview\n**CrowdAnki** also aims to facilitate collaboration by extracting all the details of an Anki deck into a single json file for easier editing.  Building on this, **anki-dm** splits this single json file into several files: one containing the raw data, one each for template layout of the cards, one for css styling, etc. allowing each of them to be edited independently.\n\nReversing the process, you can *build* a **CrowdAnki** file from these edited files and in turn *import* these files back into **Anki** with the plug-in to be used for spaced repetition memorization.\n\n## Usage\nThe usage is nearly identical to the original **anki-dm** with only slight differences to accommodate standard arg parsing in `python`.\n\n```sh\n$ python -m ankidmpy --help\nusage: anki-dm [-h] [--base BASE] [--templates]\n               {init,import,build,copy,index} ...\n\nThis tool disassembles CrowdAnki decks into collections of files and\ndirectories which are easy to maintain. It then allows you to can create\nvariants of your deck via combining fields, templates and data that you really\nneed. You can also use this tool to create translations of your deck by\ncreating localized columns in data files.\n\npositional arguments:\n  {init,import,build,copy,index}\n    init                Create a new deck from a template.\n    import              Import a CrowdAnki deck to Anki-dm format\n    build               Build Anki-dm deck into CrowdAnki format\n    copy                Make reindexed copy of Anki-dm deck.\n    index               Set guids for rows missing them.\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --base BASE           Path to the deck set directory. [Default: src]\n  --templates           List all available templates.\n$\n```\nThere are several sub-commands which each take their own options.   The `--base` switch applies to each of these sub-commands and must be supplied before the sub-command.   This switch indicates the root directory to use when looking for or generating new files.\n\nThe `--templates` switch simply lists the sample **CrowdAnki** decks which can be built upon to generate new decks and doesn\'t require a sub-command.\n\nHelp for the sub-commands can be found by applying `--help` to the sub-command:\n\n```sh\n$ python -m ankidmpy init --help\nusage: anki-dm init [-h] [--deck DECK] template\n\npositional arguments:\n  template     Template to use when creating the deck set.\n\noptional arguments:\n  -h, --help   show this help message and exit\n  --deck DECK  Name of the default deck of the deck set being created. If not\n               provided, then the original deck/template name will be used.\n$\n```\n\n## Building\n**ankidmpy** is currently written in Pure `Python` with no dependencies.  I\'ve only tried it with `python3.7` so far but it may work in earlier versions.\n\nYou can run **ankidmpy** with `python -m ankidmpy` by pointing your `PYTHONPATH` at the `src` directory or you can use [poetry](https://python-poetry.org/docs/) to build a wheel distribution like so:\n\n```sh\n$ poetry install\n$ poetry build\n```\nOnce you run `poetry install` you can also run **ankidmpy** using the **poetry** script like so:\n\n```sh\n$ poetry run anki-dm --help\n```\nSee the **poetry** documentation for more details.\n',
    'author': 'Douglas Mennella',
    'author_email': 'trx2358-pypi@yahoo.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/gitonthescene/ankidmpy',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
